/* Copyright (C) 2011, 2012, 2013 by Bundesanstalt für Gewässerkunde
 * Software engineering by Intevation GmbH
 *
 * This file is Free Software under the GNU AGPL (>=v3)
 * and comes with ABSOLUTELY NO WARRANTY! Check out the
 * documentation coming with Dive4Elements River for details.
 */

package org.dive4elements.river.exports.process;

import org.apache.log4j.Logger;
import org.jfree.data.xy.XYSeries;

import org.dive4elements.artifactdatabase.state.ArtifactAndFacet;
import org.dive4elements.artifacts.CallContext;
import org.dive4elements.river.artifacts.D4EArtifact;
import org.dive4elements.river.artifacts.model.FacetTypes;
import org.dive4elements.river.artifacts.model.WKms;
import org.dive4elements.river.artifacts.model.WQCKms;
import org.dive4elements.river.exports.StyledSeriesBuilder;
import org.dive4elements.river.exports.XYChartGenerator;
import org.dive4elements.river.exports.DiagramGenerator;
import org.dive4elements.river.jfree.StyledAreaSeriesCollection;
import org.dive4elements.river.jfree.StyledXYSeries;
import org.dive4elements.river.themes.ThemeDocument;
import org.dive4elements.river.utils.RiverUtils;

/**
 * Add data to chart/generator.
 *
 * @author <a href="mailto:bjoern.ricks@intevation.de">Björn Ricks</a>
 */
public class WOutProcessor extends DefaultProcessor {

    /** Private logger. */
    private static final Logger logger =
            Logger.getLogger(WOutProcessor.class);

    public static final String I18N_AXIS_LABEL =
        "chart.longitudinal.section.yaxis.label";

    public static final String I18N_AXIS_LABEL_DEFAULT  = "W [NN + m]";

    @Override
    public void doOut(
            DiagramGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument    theme,
            boolean          visible) {
        CallContext context = generator.getCallContext();
        Object data = bundle.getData(context);
        WKms wkms = (WKms) data;

        logger.debug("Processing facet: " + bundle.getFacetName());

        XYSeries series = new StyledXYSeries(bundle.getFacetDescription(), theme);

        if (bundle.getFacetName().equals(FacetTypes.DISCHARGE_LONGITUDINAL_C)) {
            // Add corrected values
            WQCKms wqckms = (WQCKms) data;
            int size = wqckms.size();
            for (int i = 0; i < size; i++) {
                series.add(wqckms.getKm(i), wqckms.getC(i), false);
            }
        } else {
            StyledSeriesBuilder.addPoints(series, wkms);
        }
        generator.addAxisSeries(series, axisName, visible);

        // If a "band around the curve shall be drawn, add according area.
        double bandWidth = theme.parseBandWidth();
        if (bandWidth > 0 ) {
            XYSeries seriesDown = new StyledXYSeries(
                "band " + bundle.getFacetDescription(), false, theme);
            XYSeries seriesUp = new StyledXYSeries(
                bundle.getFacetDescription()+"+/-"+bandWidth, false, theme);
            StyledSeriesBuilder.addUpperBand(seriesUp, wkms, bandWidth);
            StyledSeriesBuilder.addLowerBand(seriesDown, wkms, bandWidth);

            StyledAreaSeriesCollection area = new StyledAreaSeriesCollection(theme);
            area.addSeries(seriesUp);
            area.addSeries(seriesDown);
            area.setMode(StyledAreaSeriesCollection.FILL_MODE.BETWEEN);
            generator.addAreaSeries(area, axisName, visible);
        }

        if (bundle.getFacetName().equals(FacetTypes.LONGITUDINAL_W) ||
            bundle.getFacetName().equals(FacetTypes.DISCHARGE_LONGITUDINAL_W) ||
            bundle.getFacetName().equals(FacetTypes.STATIC_WQKMS_W) ||
            bundle.getFacetName().equals(FacetTypes.DISCHARGE_LONGITUDINAL_C)) {
            /* Only use W values to check if the diagram should be inverted
             * see flys/issue1290 for details */
            logger.debug("Check for RTL data: "+ wkms.guessRTLData());
            generator.setInverted(wkms.guessRTLData());
        }
    }

    @Override
    public void doOut(
            XYChartGenerator generator,
            ArtifactAndFacet bundle,
            ThemeDocument    theme,
            boolean          visible,
            int              index)
    {
        CallContext context = generator.getCallContext();
        WKms wkms = (WKms) bundle.getData(context);

        logger.debug("doOut");

        XYSeries series = new StyledXYSeries(bundle.getFacetDescription(), theme);

        StyledSeriesBuilder.addPoints(series, wkms);
        generator.addAxisSeries(series, index, visible);

        // If a "band around the curve shall be drawn, add according area.
        double bandWidth = theme.parseBandWidth();
        if (bandWidth > 0 ) {
            XYSeries seriesDown = new StyledXYSeries(
                "band " + bundle.getFacetDescription(), false, theme);
            XYSeries seriesUp = new StyledXYSeries(
                bundle.getFacetDescription()+"+/-"+bandWidth, false, theme);
            StyledSeriesBuilder.addUpperBand(seriesUp, wkms, bandWidth);
            StyledSeriesBuilder.addLowerBand(seriesDown, wkms, bandWidth);

            StyledAreaSeriesCollection area = new StyledAreaSeriesCollection(theme);
            area.addSeries(seriesUp);
            area.addSeries(seriesDown);
            area.setMode(StyledAreaSeriesCollection.FILL_MODE.BETWEEN);
            generator.addAreaSeries(area, index, visible);
        }

        if (bundle.getFacetName().equals(FacetTypes.LONGITUDINAL_W) ||
            bundle.getFacetName().equals(FacetTypes.DISCHARGE_LONGITUDINAL_W) ||
            bundle.getFacetName().equals(FacetTypes.STATIC_WQKMS_W)) {
            /* Only use W values to check if the diagram should be inverted
             * see flys/issue1290 for details */
            generator.setInverted(wkms.guessRTLData());
        }
    }

    @Override
    public String getAxisLabel(DiagramGenerator generator) {
        D4EArtifact flys = (D4EArtifact) generator.getMaster();

        String unit = RiverUtils.getRiver(flys).getWstUnit().getName();

        return generator.msg(
                I18N_AXIS_LABEL,
                I18N_AXIS_LABEL_DEFAULT,
                new Object[] { unit });
    }

    /**
     * Returns true if facettype is longitutinal_section.w .
     */
    @Override
    public boolean canHandle(String facetType) {
        if (facetType == null) {
            return false;
        }

        if (facetType.equals(FacetTypes.LONGITUDINAL_W)
                || facetType.equals(FacetTypes.STATIC_WKMS)
                || facetType.equals(FacetTypes.HEIGHTMARKS_POINTS)
                || facetType.equals(FacetTypes.STATIC_WQKMS)
                || facetType.equals(FacetTypes.STATIC_WQKMS_W)
                || facetType.equals(FacetTypes.DISCHARGE_LONGITUDINAL_W)
                || facetType.equals(FacetTypes.DISCHARGE_LONGITUDINAL_C)) {
            return true;
        }
        return false;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
